"""
Usage:
    udpate_package.py [<pkg>...]


"""
import socket


def check_redis():
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(10)
    try:
        sock.connect(('service.qbtrade.org', 6379))
    except:
        print('fail connect to service.qbtrade.org:6379')
        return False
    sock.close()
    return True


def check_contract():
    import requests
    r = requests.get('http://api.qbtrade.org/contracts?count=1')
    return r.status_code == 200


def check_kong():
    import requests
    r = requests.get('http://kong-admin.qbtrade.org/apis')
    return r.status_code == 200


def check_env_single():
    success = True
    for func in [check_redis, check_contract, check_kong]:
        try:
            r = func()
            if not r:
                raise Exception('check fail')
        except:
            print(func, 'fail')
            success = False
    return success


def check_env():
    import arrow
    start = arrow.now()
    t = 5
    import time
    while True:
        r = check_env_single()
        if r:
            break
        print('sleep', t)
        time.sleep(t)
        t *= 2
        t = min(t, 300)
        if (arrow.now() - start).total_seconds() > 20 * 60:
            print('too long, exit with code 1', start, arrow.now())
            exit(1)
    print('check success, time to sprint')
    # sock.settimeout(None)
    # fileobj = sock.makefile('rb', 0)


if __name__ == '__main__':
    # docopt(__doc__)

    # docopt = docoptinit(['qbtrade', 'otlib'])
    check_env()
    # update_package(['qbtrade', 'otlib'])
    # if docopt['--daemon']:
    #     pass
    # else:
    #     qb.run_until_complete(play())



import requests
from docopt import docopt as docoptinit

options = """
Usage:
    register_kong [options]
    
Options:
    --file=<file>
    --target=<target>
"""


def upload(argv):
    docopt = docoptinit(options, argv)
    file = docopt['--file']
    target = docopt['--target']
    if not target:
        index = file.rfind('/')
        if index > 0:
            target = file[index + 1:]
        else:
            target = file
    url = f'http://api.qbtrade.org/oss/public/{target}'

    requests.post(url, files={
         "file": open(file, 'rb')
    })



doc = """
Usage:
    add-version [options]
    
Options:
    --gitsync
"""


def new_date_version():
    import arrow
    new_r = str(arrow.now().date()).replace('-', '.')
    new_r = new_r.split('.')
    return '.'.join(str(int(x)) for x in new_r)


def add_version(argv):
    import os
    import re

    from . import util
    docopt = util.docopt(doc, argv)
    package = os.getcwd()
    package = os.path.split(package)[-1]
    import os
    target = os.path.join(os.getcwd(), package, '__init__.py')
    print(f'add version {target}')
    txt = open(target).read()
    r = re.findall("__version__ = '(.*)'", txt)[0]
    new_r = new_date_version()
    last = r.split('.')[-1]
    if r.startswith(new_r):
        new_r = new_r + '.' + str(int(last) + 1)
    else:
        new_r = new_r + '.0'
    print(r, new_r)
    txt = txt.replace(r, new_r)
    open(target, 'w', newline='\n').write(txt)
    if docopt['--gitsync']:
        os.system('git add . ')
        os.system('git commit -m "add version {}"'.format(new_r))
        os.system('git pull')
        os.system('git push')

    r = util.Redis.set(f'cache:pip:{package}.latest',  new_r)
    print(r.json())


if __name__ == '__main__':
    add_version([])



import os


def start(host=None):
    if host is None:
        host = os.environ.get('HOSTNAME', '')
    if '-' not in host:
        print('seems not a kube', host)
        return
    import requests
    try:
        sp = host.split('-')
        deploy = '-'.join(sp[:-2])
        if deploy == '':
            print('deploy is empty')
        print('deploy', deploy)
        import arrow
        r = requests.post('http://es.qbtrade.org/kube-start/doc',
                          json={'deploy': deploy, 'host': host, 'category': 'deploy',
                                '@timestamp': arrow.now().isoformat()},
                          timeout=3)
        print(r)
        print(r.text)
    except:
        import logging
        logging.exception('failed')


if __name__ == '__main__':
    start('pytest-argv-xx-yyy')



"""
Usage:
    qb-starter.py anytime/ashare/ctp/okcom/oddminute [options] <cmd>...

Options:
    --with-noon
    --watch-git-change=<folder>      Watch a folder or dict, and restart when something is pushed to git
    --time-expand    before only

ashare: ['weekday', '0930-1130', '1300-1500']

ashareext: ['weekday', '0915-1130', '1300-1500']
"""
import copy
import itertools
import logging
import subprocess
from collections import defaultdict
from datetime import timedelta
from time import sleep

import arrow
import psutil


class ProcessInfo:
    inst = None

    @classmethod
    def get(cls) -> 'ProcessInfo':
        if not cls.inst:
            cls.inst = ProcessInfo()
        return cls.inst

    def __init__(self):
        print('init cpu percent')
        self.p = psutil.Process()

    def mem(self, mb):
        if mb == 'mb':
            return self.p.memory_info().rss / 1024 / 1024
        else:
            raise ValueError('unit not recognize')


# p = CpuPercent()


def get_memory_mb():
    p = ProcessInfo.get()
    x = p.mem('mb')
    return x
    # print('memory:', x)


def cond_anytime(*args, **kwargs):
    logging.debug('{} {}'.format(args, kwargs))
    return True


def cond_oddminute(time=None):
    if time is None:
        time = arrow.now()
    return time.minute % 2 == 0


def wrap_expand(cond, expand):
    def func(time=None):
        if time is None:
            time = arrow.now()
        else:
            time = arrow.get(time)
        time = time.shift(seconds=expand * -1)
        return cond() if cond() else cond(time=time)

    return func


class G:
    pid = None


def kill_pid():
    print('trying to kill pid', G.pid)
    if G.pid:
        parent = psutil.Process(G.pid)
        for child in parent.children(recursive=True):
            child.kill()
            print('kill child {}'.format(child))
        parent.kill()
        print('kill parent {}'.format(parent))
        # print('kill pricess at ', gmtp8now())
        # process.kill()


def run_cmdline_when_cond(cond, cmdline, restarter):
    print('cmdline: ', cmdline)
    if cond() and restarter.do_restart():
        print('start at ', arrow.now())
        process = subprocess.Popen(cmdline)
        G.pid = process.pid
        while True:
            if process.poll() is None:
                if not cond():
                    kill_pid()
                sleep(1)
            else:
                print('program exit itself')
                break
        print('end at', arrow.now())
    else:
        cnt = 0
        while not cond(time=arrow.now().datetime + timedelta(seconds=cnt)):
            cnt += 1
        print('sleep {} seconds and restart'.format(cnt))
        sleep(cnt)
    restarter.sleep_to_next_possible()


class Restarter:
    def __init__(self, min_interval=1, max_interval=10):
        self.min_interval = min_interval
        self.max_interval = max_interval
        self.bad_interval = []  # [(begin, end)...]  代表程序有问题的区间

    def clean_old_bad_interval(self, time):
        while self.bad_interval:
            b, e = self.bad_interval[0]
            if e < time.shift(minutes=-self.max_interval):
                self.bad_interval.pop(0)
            else:
                break

    def count_total_bad_interval_minutes(self, time):
        interval = 0
        for b, e in copy.deepcopy(self.bad_interval):
            interval += (e - max(b, time.shift(minutes=-self.max_interval))).total_seconds() / 60
        return max(interval, self.min_interval)

    def do_restart(self, time=None):
        time = time or arrow.now()
        if self.can_restart(time):
            end = time.shift(minutes=self.count_total_bad_interval_minutes(time))
            self.bad_interval.append((time, end))
            return True
        return False

    def get_last_bad_interval(self):
        if self.bad_interval:
            return self.bad_interval[-1][1]
        else:
            return None

    def can_restart(self, time):
        if not self.bad_interval:
            return True
        self.clean_old_bad_interval(time)
        if self.get_last_bad_interval() is None or time >= self.get_last_bad_interval():
            return True
        return False

    def sleep_to_next_possible(self, time=None):
        time = time or arrow.now()
        for i in itertools.count():  # for i in range( +inf)
            if self.can_restart(time.shift(seconds=i)):
                logging.info('sleep {} until next possible'.format(i))
                logging.info(self.bad_interval)
                sleep(i)
                return


def supervisor(argv):
    print(argv)
    args = argv[:]
    start = args[0]
    args = args[1:]
    time_expand = 0
    options = defaultdict(bool)
    cmdline = ''
    for i, item in enumerate(args):
        if not item.startswith('-'):
            if args[i - 1] == '--time-expand':
                time_expand = float(item)
                continue
            else:
                cmdline = args[i:]
                break

        else:
            options[item] = True

    if start == 'anytime':
        cond = cond_anytime
    elif start == 'oddminute':
        cond = cond_oddminute
    else:
        logging.warning('start time range not recognized')
        assert False
    # cur = arrow.now().datetime
    # for _ in range(1000):
    #     cur += timedelta(minutes=5)
    #     print(cur, cond(cur))
    # exit(0)
    restarter = Restarter()
    cond = wrap_expand(cond, time_expand)
    while True:
        try:
            run_cmdline_when_cond(cond, cmdline, restarter)
        except KeyboardInterrupt:
            print('KeyboardInterrupt catch, kill all and exit')
            kill_pid()
            break


if __name__ == '__main__':
    supervisor(['oddminute', 'bash'])



"""
Usage:
    pull [options]

Options:
    --image=<img>

"""

import threading

import requests


def pull_single(host, image):
    # print('pull image', host)
    try:
        x = requests.post('http://{}:5748/pull'.format(host),
                          data={'image': image},
                          timeout=(5, 600))
        print(x.text)
    except requests.exceptions.ConnectTimeout:
        print('timeout', host, 'server may not start')
    except Exception as e:
        print('error', host, e)
        # logging.exception('unexpected fail')
    else:
        print('pull image done', host)


# haproxy --port port --dest host:port
def pull(argv):
    from docopt import docopt as docoptinit

    docopt = docoptinit(__doc__, argv)
    print(docopt)
    for i in range(1, 20):
        host = 'alihk-otslave-{}'.format(i)
        threading.Thread(target=pull_single, args=(host, docopt['--image'])).start()


if __name__ == '__main__':
    pull(['--image', 'registry.cn-hangzhou.aliyuncs.com/qbtrade/backend-go'])



def test_watch():
    from qbx import watch_http
    r = watch_http.check('api.qbtrade.org')
    assert not r

    r = watch_http.check('api.qbtrade.org/v1/basic/time')
    assert r

    r = watch_http.check('xxx.qbtrade.org/v1/basic/time')
    assert r


def test_timeout():
    from qbx import watch_http
    import pytest
    with pytest.raises(SystemExit):
        watch_http.watch_http(['qadfaf.afafasfsadfasdfasf.com', '--timeout', '3'])



# Super simple script that listens to a local UDP port and relays all packets to an arbitrary remote host.
# Packets that the host sends back will also be relayed to the local UDP client.
# Works with Python 2 and 3

import sys
import socket
from docopt import docopt as docoptinit

uproxy_doc = """
Usage:
    uproxy.py [options]

Options:
    --port=<port>                local port
    --dest=<dest>                dest format as host:port
"""


def fail(reason):
    sys.stderr.write(reason + '\n')
    sys.exit(1)


# uproxy --port port --dest host:port
def uproxy(argv):
    docopt = docoptinit(uproxy_doc, argv)
    print(docopt)
    port = int(docopt['--port'])
    dest = docopt['--dest']
    dest_host, dest_port = dest.split(':')
    dest_port = int(dest_port)

    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.bind(('', port))
    except:
        fail('Failed to bind on port ' + str(port))

    known_server = (dest_host, dest_port)
    sys.stderr.write('All set.\n')
    while True:
        data, addr = s.recvfrom(32768)
        print('recv', data)
        s.sendto(data, known_server)


if __name__ == '__main__':
    uproxy(['--port', '12637', '--dest', 'localhost:3333'])



import sys

from docopt import docopt as docoptinit


def must_success(func, *args, **kwargs):
    if 'timeout' not in kwargs:
        kwargs['timeout'] = 5
    try:
        r = func(*args, **kwargs)
    except Exception as e:
        print('force exit, statuscode != 200', type(e))
        sys.exit(1)
    if r.status_code >= 300:
        print(r, r.text)
        print('force exit, statuscode != 200')
        sys.exit(1)
    return r


def docopt(doc, argv):
    return docoptinit(doc, argv=argv)


class Redis:
    inst = None

    @classmethod
    def __get_instance(cls):
        import redis
        if cls.inst is None:
            cls.inst = redis.StrictRedis(host='redis.svc.qbtrade.org', socket_connect_timeout=5, socket_timeout=5,
                                         decode_responses=True)
        return cls.inst

    @classmethod
    def get(cls, key):
        return cls.__get_instance().get(key)

    @classmethod
    def set(cls, key, value):
        return cls.__get_instance().set(key, value)


if __name__ == '__main__':
    r = Redis.get('config:v1ws')
    print(r)
    r = Redis.set('test:config:v1ws', 'hello-world')
    print(r)



import logging
import os
import sys

help_doc = """
usage: qbx [--help] 
           <command> [<args>]

These are some qbx commands:
    auth_wrapper
    register-kong
    register-ws
    watch_git
    watch_git_http
    build-curdir
    watch-http
    pull
    haproxy
    uproxy
    update-package
    add-version
    check-env
"""


def print_help():
    print(help_doc)


def warning(message, seconds=10):
    import time
    logging.warning(message + f'sleep {seconds}s to let you change')
    for i in range(10, 0, -1):
        print(i, 'seconds left')
        time.sleep(1)


def run():
    logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)
    print('argv---', sys.argv)
    if len(sys.argv) == 1:
        print_help()
    else:
        argv = sys.argv[2:]
        if sys.argv[1] == 'register-kong':
            from .register_kong import register_kong
            register_kong(argv)
        elif sys.argv[1] == 'register_kong':
            warning('register_kong is deprecated use "qbx register-kong" instead')
            from .register_kong import register_kong
            register_kong(argv)
        elif sys.argv[1] == 'register_ws':
            from .register_ws import register_ws
            warning('register_ws is deprecated use "qbx register-ws" instead')
            register_ws(argv)
        elif sys.argv[1] == 'register-ws':
            from .register_ws import register_ws
            register_ws(argv)
        elif sys.argv[1] == 'build-curdir':
            from .build_curdir import build_curdir
            build_curdir(argv)
        elif sys.argv[1] == 'watch_git':
            from .watchgit import watch_git
            watch_git(argv)
        elif sys.argv[1] == 'watch_git_http':
            from .watchgit import watch_git_http
            watch_git_http(argv)
        elif sys.argv[1] == 'haproxy':
            from .haproxy import haproxy
            haproxy(argv)
        elif sys.argv[1] == 'pull':
            from .pull import pull
            pull(argv)
        elif sys.argv[1] == 'uproxy':
            from .uproxy import uproxy
            uproxy(argv)
        elif sys.argv[1] == 'auth_wrapper':
            from .auth_wrapper import auth_wrapper
            auth_wrapper(argv)
        elif sys.argv[1] == 'kube-start':
            from . import kube_start
            kube_start.start()
        elif sys.argv[1] == 'upload':
            from .upload import upload
            upload(argv)
        elif sys.argv[1] == 'update-package':
            from .update_package import update_package
            update_package(argv)
        elif sys.argv[1] == 'check-env':
            from .check_env import check_env
            check_env()
        elif sys.argv[1] == 'add-version':
            from .add_version import add_version
            add_version(argv)
        elif sys.argv[1] == 'watch-http':
            from .watch_http import watch_http
            watch_http(argv)
        else:
            if sys.argv[1] != 'help' and sys.argv[1] != '--help':
                logging.warning('method not regognize')
            print_help()


def qbs():
    print(sys.argv)
    print(os.environ)
    run_cmdline_when_cond(sys.argv[1:])


def run_cmdline_when_cond(cmdline):
    print('cmdline: ', cmdline)
    import subprocess
    import time
    import arrow
    process = subprocess.Popen(cmdline)
    while True:
        if process.poll() is None:
            time.sleep(1)
        else:
            print('program exit itself')
            break
    print('end at', arrow.now(), 'sleep 30s before exit')
    time.sleep(30)


if __name__ == '__main__':
    # from .watchgit import watch_git
    # watch_git(['git+ssh://git@github.com/qbtrade/quantlib.git', 'log_rpc.py'])
    from . import kube_start

    kube_start.start()



import arrow

from .cdi import docker_retag_and_push, get_docker_client

doc = """
Usage:
    build-curdir [options]
    
Options:
    --repo=<repo>
    --no-cache
"""


def build_curdir(argv):
    import os
    from docopt import docopt as docoptinit
    import json
    import sys
    docopt = docoptinit(doc, argv)
    folder = os.path.abspath('.')

    # parent_hash = get_hash_of_dirs(folder)
    # if not docopt['--always-rebuild']:
    #     if folder in db and db[folder] == parent_hash:
    #         print('{} hash check sum'.format(folder))
    #         return
    success = False
    if docopt['--repo']:
        repo = docopt['--repo']
    else:
        repo = os.path.basename(folder)
    tag = 'qbtrade/{}:latest'.format(repo)
    print('------build------', folder, tag)
    for line in get_docker_client().build(path=folder,
                                          dockerfile='Dockerfile',
                                          nocache=docopt['--no-cache'],
                                          tag=tag,
                                          rm=True):
        try:
            line = line.decode('utf8')
            print(json.loads(line)['stream'], end='')
        except:
            print(line, end='')
        if 'Successfully built' in line:
            success = True
    if not success:
        sys.exit(1)
    print('------push------')
    # cfg = load_cdi_config(folder)
    # if cfg.get('public', True):
    #     docker_push(tag)
    #     docker_retag_and_push(tag, tag + '-' + date_str)
    # else:
    #     print('only push to private repo')
    if os.environ.get('QB_REGION') == 'alihk':
        docker_retag_and_push_all(tag, 'registry-vpc.cn-hongkong.aliyuncs.com/' + tag)
    else:
        docker_retag_and_push_all(tag, 'registry.cn-hongkong.aliyuncs.com/' + tag)


def docker_retag_and_push_all(tag, param):
    # push all qbtrade/ot-rpc:latest registry-vpc.cn-hongkong.aliyuncs.com/qbtrade/ot-rpc:latest
    print('push all', tag, param)
    date_str = arrow.now().strftime('%Y%m%d')
    datetime_str = arrow.now().strftime('%Y%m%d-%H%M')
    docker_retag_and_push(tag, param)
    if param.endswith(':latest'):
        docker_retag_and_push(tag, param.replace('latest', '') + date_str)
        docker_retag_and_push(tag, param.replace('latest', '') + datetime_str)
    else:
        docker_retag_and_push(tag, param + '-' + date_str)
        docker_retag_and_push(tag, param + '-' + datetime_str)

    # docker_retag_and_push_all(tag, 'registry.cn-hangzhou.aliyuncs.com/' + tag)

    # db[folder] = parent_hash



import arrow
import requests
from docopt import docopt as docoptinit

watch_http_doc = """
Usage:
    watch_http <url> [options]

Options:
    --timeout=<t>   Timeout [default: 30]
"""


def check(url):
    if not url.startswith('http://'):
        url = 'http://' + url
    try:
        r = requests.get(url, timeout=1)
        if r.status_code == 200:
            return True
        return False
    except:
        return False


def watch_http(argv):
    print(watch_http_doc)
    print(argv)
    docopt = docoptinit(watch_http_doc, argv)
    print(docopt)
    url = docopt['<url>']
    timeout = int(docopt['--timeout'])
    start = arrow.now()
    while arrow.now() < start.shift(seconds=timeout):
        if check(url):
            return True
        print('check', url, 'fail')
        import time
        time.sleep(1)
    import sys
    print('fail watch http')
    sys.exit(1)


def play():
    watch_http(['www.baidu.com', '--timeout', '10'])
    watch_http(['qadfaf.afafasfsadfasdfasf.com', '--timeout', '10'])
    # watch_http(['www.baidu.com'])


if __name__ == '__main__':
    play()



import asyncio
from aiohttp import web
from aiohttp.web import run_app
from docopt import docopt as docoptinit
import os
import tempfile
import json
import logging

watch_git_doc = """
Usage:
    watch_git.py [options] <repo> <path>
    
Options:
    --debug

"""


def init_repo(directory, repo):
    os.makedirs(directory, exist_ok=True)
    os.system('cd {} && git clone {} watch'.format(directory, repo))


def watch_git(argv):
    docopt = docoptinit(watch_git_doc, argv)
    directory = tempfile.mkdtemp()
    print('watch', directory, docopt)
    init_repo(directory, docopt['<repo>'])
    path = '{}/watch/{}'.format(directory, docopt['<path>'])
    with open('{}/watch/{}'.format(directory, docopt['<path>'])) as f:
        buf = f.read()
    while True:
        import time
        time.sleep(10)
        os.system('cd {}/watch && git pull'.format(directory))
        with open(path) as f:
            new = f.read()
        if new != buf:
            logging.warning('changed')
            break


watch_git_http_doc = """
Usage:
    watch_git_http [options] <repo> <path>

Options:
    --port=<port>  Port [default: 3000]
    --debug        Debug or not

"""


def watch_git_http(argv):
    docopt = docoptinit(watch_git_http_doc, argv)
    logging.info('watch http git')
    logging.info(docopt)
    directory = tempfile.mkdtemp()
    print('watch', directory, docopt)
    os.system('cd {} && git clone {} watch'.format(directory, docopt['<repo>']))
    path = '{}/watch/{}'.format(directory, docopt['<path>'])
    with open('{}/watch/{}'.format(directory, docopt['<path>'])) as f:
        buf = f.read()

    def jsonify(dic, status=200):
        text = json.dumps(dic, sort_keys=True)
        return web.Response(body=text.encode('utf-8'), content_type='application/json', status=status)

    @asyncio.coroutine
    def check_repo(request):
        logging.info('checking')
        os.system('cd {}/watch && git pull'.format(directory))
        with open(path) as f:
            new = f.read()
        if new != buf:
            logging.warning('changed')
            return jsonify({'message': 'repo changed'}, 500)
        logging.info('not changed')
        return jsonify({'message': 'repo not change'})

    app = web.Application()
    app.router.add_route('GET', '/watchgit', check_repo)
    run_app(app, port=int(docopt['--port']))



import asyncio
import sys

import jwt
from aiohttp import web, ClientSession
from docopt import docopt as docoptinit

bad_headers = ()

auth_jwt_doc = """
Usage:
    auth_wrapper [options]
    
Options:
    --port=<port>               listen port
    --dest=<dest>               upstream as form as {host}:{port}
    --debug                     using debug pub and private
    --name=<name>               server symbol
    --noauth                    Ignore jwt auth, for debug purpose
"""

debug_pub = """
-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDvWEO0amFTUXJdF6TsevXNfxEg
is9YfaRzAVef9Dm3CeJKdXqS/KCP1/8iw2tcRWGEY1MiCwp3cegwujWnDr869cCh
FccrhRcWEVwCPEioosvMTajJSgn0nGGZf6HC/qDMT1ESiD0VlWa1miF457mXXbS0
/jKAYJBiCAKU8g+OZQIDAQAB
-----END PUBLIC KEY-----
"""

debug_pri = """
-----BEGIN RSA PRIVATE KEY-----
MIICXQIBAAKBgQDvWEO0amFTUXJdF6TsevXNfxEgis9YfaRzAVef9Dm3CeJKdXqS
/KCP1/8iw2tcRWGEY1MiCwp3cegwujWnDr869cChFccrhRcWEVwCPEioosvMTajJ
Sgn0nGGZf6HC/qDMT1ESiD0VlWa1miF457mXXbS0/jKAYJBiCAKU8g+OZQIDAQAB
AoGAdAS9DP9kHhck8Ks9bsRL0kj97GBdEfAVfwnvh8HDGE7aOm2n9QgwbImvSxKf
QCMBmkLrUV04vZ2hh707tLcZSnM/ulOkLGsytPruM5h9cnV3lhsNLt4jtuSlDvVz
j9Z4mxXOVidjNq41Pg4qKDQS8q/vB+x7DtEn/4zvBotr1QECQQD8aswh5yn9RDao
rMiNmHmh2ldSarbJcZFzsQoX9CK3el+oo7XRguBglwRFHDBNIAh92fUiX4KATZRF
QrIzQnhJAkEA8r33Wd/gZj5GH+AJ/K9zYU4yXYdbMdwaHU/D/XrpfxqtxqQ9h6e4
pNLyOStEnwXPD4voL92aLOwW5Dg34xO9PQJBAKphOEsGK0yeV7rBblpNeoSqydiC
2cDd3M1XyjVjAHAStTEy2A6EpgnsxeAUZ/IXVkQE9Ddwerk6JIQfwgNhsakCQAT5
bZsy4kdWGVvH3IyID+Y7kv6lqnHAH+zf2JVWMni/VDZQ4U3pWvhNtlcDkvlrRg38
gPqSIPmwsNtmZ4bIvcUCQQDRybMD4BRDiZm8JG/gKwHwolUUCKtxjljwUiAq6J+A
t/7kQ94wZnOQDADVbcHs9vU/YRvBHUxF9Uky/Gs+Uzm9
-----END RSA PRIVATE KEY-----
"""

prod_pub = """
-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAptDAATPBIAhmbuU5uJiH
Q/h3alX55fMHPynmxdXgvrPIg5Xv5VXuJBBCR99Tol5Wd/oiln0t+2Z167zy06GA
82Nb4Q/WggWOZGdp5asSihJnbN9eDWqo2J4ppyh9DBXgVB5OS1aaosbCIj0vnY8z
CVlFcr854ohllv7HYK1tTQadhnax02WvJJYPRyzITTHVtO99abpoMKj8GlpcZJ8d
Oe9Iy5h+i1wUCSulg/e4id9P0XBSN5iy1LcwL6BEj72qTA+egah0LrDpzHUnIMo3
+pniYhsJIlD8iSzICi5T2ca155zMsPv0KtuRii4TT21I3V9qY+G6Qwbe2BjZO3N0
JQIDAQAB
-----END PUBLIC KEY-----
"""


def start_proxy(port, dest, pub, name, noauth=False):
    def remove_bad_headers(headers):
        h = {}
        for key, value in headers.items():
            if key.lower() not in bad_headers:
                h[key] = value
        # h['Connection'] = "close"
        return h

    async def jwt_check(token):
        if noauth:
            return True
        print('check', token)
        if token is not None:
            payload = jwt.decode(token, key=pub, algorithms='RS256')
            print(f'name: {name}, path: {payload["permissions"]}')
            if name in payload['permissions']:
                return True
        return False

    async def auth_jwt(app, handler):
        async def do_it(request):
            print("==> %s" % dest)
            method = request.method
            path_qs = request.path_qs
            headers = remove_bad_headers(request.headers)
            cookies_str = headers.get('Cookie', '')
            data = await request.read()
            params = dict(request.query)
            cookies = {}
            for item in cookies_str.split(';'):
                sp = item.split('=')
                if len(sp) == 2:
                    cookies[sp[0].strip()] = sp[1].strip()
            print(cookies)
            auth = await jwt_check(cookies.get('jwt'))

            if not auth:
                r = {'code': 'auth-failed'}
                import json
                return web.Response(text=json.dumps(r), status=403)
            url = dest + path_qs
            if not url.startswith('http'):
                url = 'http://' + url

            print(app.loop, method, url, headers, data, params)
            return await get_resp(app.loop, method, url=url, headers=headers, data=data, params=params)

        return do_it

    async def get_resp(loop, method, url, headers=None, data=None, params=None):
        async with ClientSession(loop=loop) as session:
            async with session.request(method, url, headers=headers, data=data, params=params) as resp:
                body = await resp.read()
                headers = remove_bad_headers(resp.headers)
                # print(f"<== {method} {url} {dest} {resp.status} {body}")
                response = web.Response(body=body, status=resp.status, reason=resp.reason, headers=headers)
        return response

    async def init(loop, port):
        app = web.Application(loop=loop, middlewares=[auth_jwt])
        return await loop.create_server(app.make_handler(), "", port)

    def start():
        loop = asyncio.get_event_loop()
        loop.run_until_complete(init(loop, port))
        loop.run_forever()

    start()


def auth_wrapper(argv):
    docopt = docoptinit(auth_jwt_doc, argv)
    print(docopt)
    port = int(docopt['--port'])
    dest = docopt['--dest']
    name = docopt['--name']
    if not dest.startswith('http'):
        raise Exception('--dest should starts with http;')
    if not name:
        raise Exception('--name must have')

    if docopt['--debug']:
        pub = debug_pub
    else:
        pub = prod_pub
    start_proxy(port, dest, pub, name, docopt['--noauth'])


if __name__ == "__main__":
    auth_wrapper(['--port', '3000', '--dest', 'http://konga.qbtrade.org', '--name', 'konga', '--noauth'])
    # auth_wrapper(['--port', '3000', '--dest', 'http://konga.qbtrade.org', '--name', 'konga', '--debug'])
    # auth_wrapper(sys.argv[1:])



from setuptools import setup

setup(name='qbx',
      author='qbtrade',
      url='https://github.com/qbtrade/qbx',
      author_email='admin@qbtrade.org',
      packages=['qbx'],
      version='2018.12.3.2',
      description='Qbtrade Management Script',
      install_requires=[
          'docopt',
          'requests',
          'flask',
          'flask_restful',
          'redis',
      ],
      zip_safe=False,
      entry_points={'console_scripts': ['qbx=qbx:run', 'qbs=qbx:qbs']}
      )



"""
Usage:
    udpate_package.py [<pkg>...]


"""
import redis
import socket


def check_dns():
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(10)
    # r = sock.connect(('service.qbtrade.org', 6379))
    try:
        sock.connect(('service.qbtrade.org', 6379))
    except ConnectionError:
        print('fail connect to service.qbtrade.org:6379')
        return False
    sock.close()
    return True
    # sock.settimeout(None)
    # fileobj = sock.makefile('rb', 0)


def sleep_until_check_pass():
    sleep = 60
    import time
    while True:
        try:
            if check_dns():
                return
            print('check fail sleep {}'.format(sleep))
            sleep = min(300, sleep * 2)
            time.sleep(sleep)
        except:
            import logging
            logging.exception('unexpected failed')
            import time
            time.sleep(300)
            print('sleep 300s and panic')
            import os
            # noinspection PyProtectedMember
            os._exit(1)


def check_package(pkg='qbtrade'):
    r = redis.StrictRedis(host='service.qbtrade.org', socket_connect_timeout=1, socket_timeout=1, decode_responses=True)
    y = r.get(f'cache:pip:{pkg}.latest')
    import pkg_resources

    try:
        x = pkg_resources.get_distribution(pkg)
        version = x.version
    except:
        version = 'not-exist'
    if version != y:
        import os
        print(f'update version from {version} to {y}')
        os.system(f'pip install http://api.qbtrade.org/oss/public/{pkg}-latest-py3-none-any.whl -U --no-deps')


def update_package(argv):
    from docopt import docopt as docoptinit
    docopt = docoptinit(__doc__, argv)
    # print(docopt)
    # sleep_until_check_pass()
    pkg = docopt['<pkg>']
    if not pkg:
        pkg = ['qbtrade', 'otlib']
    print('update package', pkg)
    for p in pkg:
        check_package(p)
        # check_package('otlib')


if __name__ == '__main__':
    # docopt(__doc__)

    # docopt = docoptinit(['qbtrade', 'otlib'])
    update_package([])
    # update_package(['qbtrade', 'otlib'])
    # if docopt['--daemon']:
    #     pass
    # else:
    #     qb.run_until_complete(play())
